@@ -1,3 +1,4 @@
+#![allow(clippy::redundant_clone)] // due to a structopt problem
 use std::io::stdout;
 
 use anyhow::{Context, Result};
@@ -50,24 +51,26 @@ fn setup_logger(verbose: bool) -> Result<(), fern::InitError> {
 
 fn main() -> Result<()> {
     let args: Cli = Cli::from_args();
+    setup_logger(args.verbose).context("Failed to setup the logger")?;
+    debug!("{:?}", args);
+
     args.validate_input_output_combination()?;
     let is_illumina = args.input.len() == 2;
     if is_illumina {
         info!("Two input files given. Assuming paired Illumina...")
     }
 
-    setup_logger(args.verbose).context("Failed to setup the logger")?;
-
-    debug!("{:?}", args);
-
     let input_fastx = Fastx::from_path(&args.input[0]);
 
     let mut output_handle = match args.output.len() {
-        0 => Box::new(stdout()),
+        0 => match args.output_type {
+            None => Box::new(stdout()),
+            Some(fmt) => niffler::basic::get_writer(Box::new(stdout()), fmt, args.compress_level)?,
+        },
         _ => {
             let out_fastx = Fastx::from_path(&args.output[0]);
             out_fastx
-                .create()
+                .create(args.compress_level, args.output_type)
                 .context("unable to create the first output file")?
         }
     };
@@ -112,27 +115,30 @@ fn main() -> Result<()> {
         seed: args.seed,
     };
 
-    let reads_to_keep = subsampler.indices(&read_lengths);
+    let (reads_to_keep, nb_reads_to_keep) = subsampler.indices(&read_lengths);
     if is_illumina {
-        info!("Keeping {} reads from each input", reads_to_keep.len());
+        info!("Keeping {} reads from each input", nb_reads_to_keep);
     } else {
-        info!("Keeping {} reads", reads_to_keep.len());
+        info!("Keeping {} reads", nb_reads_to_keep);
     }
     debug!("Indices of reads being kept:\n{:?}", reads_to_keep);
 
     let mut total_kept_bases =
-        input_fastx.filter_reads_into(reads_to_keep.clone(), &mut output_handle)? as u64;
+        input_fastx.filter_reads_into(&reads_to_keep, nb_reads_to_keep, &mut output_handle)? as u64;
 
     // repeat the same process for the second input fastx (if illumina)
     if is_illumina {
         let second_input_fastx = Fastx::from_path(&args.input[1]);
         let second_out_fastx = Fastx::from_path(&args.output[1]);
         let mut second_output_handle = second_out_fastx
-            .create()
+            .create(args.compress_level, args.output_type)
             .context("unable to create the second output file")?;
 
-        total_kept_bases +=
-            second_input_fastx.filter_reads_into(reads_to_keep, &mut second_output_handle)? as u64;
+        total_kept_bases += second_input_fastx.filter_reads_into(
+            &reads_to_keep,
+            nb_reads_to_keep,
+            &mut second_output_handle,
+        )? as u64;
     }
 
     let actual_covg = total_kept_bases / args.genome_size;
